const mongoose = require('mongoose');
const express = require('express');
const cookieParser = require('cookie-parser');
const fs = require('fs');
const crypto = require('crypto');
const cm = require('./customsessions');

cm.sessions.startCleanup();

const connection_string = 'mongodb://127.0.0.1/responsibilities';

mongoose.connect(connection_string);
mongoose.connection.on('error', () => {
  console.log('There was a problem connecting to mongoDB');
});

/**
 * Specify the schema for the database, including items and categories.
 */
var UserSchema = new mongoose.Schema( {
    username: String,
    salt: Number,
    hash: String,
    favoriteColors: [String]
});
var User = mongoose.model('User', UserSchema);
var ItemSchema = new mongoose.Schema( {
    name: String,
    difficulty: Number,
    stat: String,
    category: String
});
var Item = mongoose.model('Item', ItemSchema);
var CategorySchema = new mongoose.Schema( {
  name: String,
  priority: Number,
  color: String
});
var Category = mongoose.model('Category', CategorySchema);

function authenticate(req, res, next) {
  let c = req.cookies;
  if (c && c.login) {
    let result = cm.sessions.doesUserHaveSession(c.login.username, c.login.sid);
    if (result) {
      next();
      return;
    }
  }
  res.redirect('/account/index.html');
}

/**
 * Initialize the express app and configure with various features 
 * such as JSON parsing, static file serving, etc.
 */
const app = express();
app.use(cookieParser());    
app.use('/app/*', authenticate);
app.use(express.static('public_html'))
//app.get('/', (req, res) => { res.redirect('/app/index.html'); });
app.use(express.json())
//app.use(parser.text({type: '*/*'}));
        

app.use('*', (req, res, next) => {
  let c = req.cookies;
  if (c && c.login) {
    if (cm.sessions.doesUserHaveSession(c.login.username, c.login.sid)) {
      cm.sessions.addOrUpdateSession(c.login.username);
    }
  }
  next();
});



/**
 * Get all of the items that correspond with a particular catagory.
 * The category is specified as a part of the URL after /items/.
 */
app.get('/items/:cat', (req, res) => {
    let cat = req.params.cat;
    let p1 = Item.find({category: cat}).exec();
    p1.then( (results) => { 
      res.end( JSON.stringify(results) );
    });
    p1.catch( (error) => {
      console.log(error);
      res.end('FAIL');
    });
});

app.get('/testing', (req, res, next) => {
  console.log('A');
  next();
});

app.get('/testing', (req, res, next) => {
  console.log('B');
  next();
});

app.get('/testing', (req, res) => {
  console.log('C');
  res.end('hi')
});


/**
 * This route is for creating a new item for a particular category. 
 * The category that the items belongs to is stored as a part of the item object in the database.
 */
app.post('/create/item/', (req, res) => {
  let newItemToSave = req.body;
  var newItem = new Item(newItemToSave);
  let p1 = newItem.save();
  p1.then( (doc) => { 
    res.end('SAVED SUCCESFULLY');
  });
  p1.catch( (err) => { 
    console.log(err);
    res.end('FAIL');
  });
});

/**
 * This route is for creating a new list category.
 */
app.post('/create/category/', (req, res) => {
  let CategoryToSave = req.body;
  var newCategory = new Category(CategoryToSave);
  let p1 = newCategory.save();
  p1.then( (doc) => { 
    res.end('SAVED SUCCESFULLY');
  });
  p1.catch( (err) => { 
    console.log(err);
    res.end('FAILED TO CREATE A CATEGORY');
  });
});

/**
 * This route is for fetching all of the categories stored in the database.
 */
app.get('/categories/', (req, res) => {
    let p1 = Category.find({}).exec();
    p1.then( (results) => { 
      res.end( JSON.stringify(results) );
    });
    p1.catch( (error) => {
      console.log(error);
      res.end('FAIL');
    });
});

/**
 * This route is for creating a new user account.
 */
app.get('/account/create/:username/:password', (req, res) => {
  let p1 = User.find({username: req.params.username}).exec();
  p1.then( (results) => { 
    if (results.length > 0) {
      res.end('That username is already taken.');
    } else {

      let newSalt = Math.floor((Math.random() * 1000000));
      let toHash = req.params.password + newSalt;
      var hash = crypto.createHash('sha3-256');
      let data = hash.update(toHash, 'utf-8');
      let newHash = data.digest('hex');

      var newUser = new User({ 
        username: req.params.username,
        salt: newSalt,
        hash: newHash
      });
      newUser.save().then( (doc) => { 
          res.end('Created new account!');
        }).catch( (err) => { 
          console.log(err);
          res.end('Failed to create new account.');
        });
    }
  });
  p1.catch( (error) => {
    res.end('Failed to create new account.');
  });
});

/**
 * Process a user login request.
 */
app.post('/account/login/', (req, res) => {
  let u = req.body.username;
  let p = req.body.password;
  console.log(u);
  console.log(p);
  let p1 = User.find({username:u}).exec();
  p1.then( (results) => {
    console.log(results);
    for(let i = 0; i < results.length; i++) {

      let existingSalt = results[i].salt;
      let toHash = req.body.password + existingSalt;
      var hash = crypto.createHash('sha3-256');
      let data = hash.update(toHash, 'utf-8');
      let newHash = data.digest('hex');
      
      if (newHash == results[i].hash) {
        let id = cm.sessions.addOrUpdateSession(u);
        res.cookie("login", {username: u, sid: id}, {maxAge: 60000*60*24});
        res.end('SUCCESS ' + JSON.stringify(results));
        return;
      } 
    } 
    res.end('login failed');
  });
  p1.catch( (error) => {
    res.end('login failed');
  });
});

// Start up the server to listen on port 80
const port = 80;
app.listen(port, () => { console.log('server has started'); });

